# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from wds_client.api.general_wds_information_api import GeneralWDSInformationApi


class TestGeneralWDSInformationApi(unittest.TestCase):
    """GeneralWDSInformationApi unit test stubs"""

    def setUp(self) -> None:
        self.api = GeneralWDSInformationApi()

    def tearDown(self) -> None:
        pass

    def test_status_get(self) -> None:
        """Test case for status_get

        Gets health status for WDS -- generated via Spring Boot Actuator (see https://docs.spring.io/spring-boot/docs/current/actuator-api/htmlsingle/#health for details)
        """
        pass

    def test_version_get(self) -> None:
        """Test case for version_get

        Gets related git and build version info for WDS -- generated via Spring Boot Actuator (see https://docs.spring.io/spring-boot/docs/current/actuator-api/htmlsingle/#info for details)
        """
        pass


if __name__ == '__main__':
    unittest.main()
