# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class SearchSortDirection(str, Enum):
    """
    Sort direction (descending or ascending) 
    """

    """
    allowed enum values
    """
    ASC = 'ASC'
    DESC = 'DESC'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SearchSortDirection from a JSON string"""
        return cls(json.loads(json_str))


