# coding: utf-8

"""
    Workspace Data Service

    This page lists current APIs.  All v0.2 APIs are subject to change without notice.  Changelog at [https://github.com/DataBiosphere/terra-workspace-data-service/releases](https://github.com/DataBiosphere/terra-workspace-data-service/releases) 

    The version of the OpenAPI document: v0.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from wds_client.api.collection_api import CollectionApi


class TestCollectionApi(unittest.TestCase):
    """CollectionApi unit test stubs"""

    def setUp(self) -> None:
        self.api = CollectionApi()

    def tearDown(self) -> None:
        pass

    def test_create_collection_v1(self) -> None:
        """Test case for create_collection_v1

        Create a collection in this workspace.
        """
        pass

    def test_delete_collection_v1(self) -> None:
        """Test case for delete_collection_v1

        Delete the specified collection.
        """
        pass

    def test_get_collection_v1(self) -> None:
        """Test case for get_collection_v1

        Retrieve a single collection.
        """
        pass

    def test_list_collections_v1(self) -> None:
        """Test case for list_collections_v1

        List all collections in this workspace.
        """
        pass

    def test_update_collection_v1(self) -> None:
        """Test case for update_collection_v1

        Update the specified collection.
        """
        pass


if __name__ == '__main__':
    unittest.main()
